package dht;

import java.net.*;
import java.nio.file.Files;
import java.io.*;

import javax.swing.JOptionPane;

public class FileReceiver implements Runnable{

	private String fileName;
	private String address;
	private int port;
	private DHTNetwork dht;
	
	public FileReceiver(String fileName, String address, int port, DHTNetwork dht)
	{
		this.fileName = fileName;
		this.address = address;
		this.port = port;
		this.dht = dht;
	}
	
	@Override
	public void run() {
		try
		{
			//Important that we use stream socket so that we can actually
			//receive a file. DatagramSockets were only useful for packets because
			//information order and reliability was not as important.
			Socket recvSocket = new Socket(address, port);
			byte[] byteArray = new byte[1024];
			int bytesRead;
					
			InputStream input = recvSocket.getInputStream();
			FileOutputStream fileOutput = new FileOutputStream(fileName);
			BufferedOutputStream bufferedOutput = new BufferedOutputStream(fileOutput);
			recvSocket.setSoTimeout(10000);
					
			bytesRead = input.read(byteArray, 0, byteArray.length);
			if(bytesRead != -1)
			{
				bufferedOutput.write(byteArray, 0, bytesRead);
			}
			bufferedOutput.close();
			fileOutput.close();
			recvSocket.close();
			JOptionPane.showMessageDialog(null, "Finished receiving file: " + fileName + "(" + dht.hash(fileName) + ").");
			System.out.println("Finished receiving file: " + fileName + " (" + dht.hash(fileName) + ").");
		}
		catch(IOException ioException)
		{
			System.gc();
			JOptionPane.showMessageDialog(null, "File: " + fileName + " (" + dht.hash(fileName) + ") could not be received.");
			System.out.println("File: " + fileName + " (" + dht.hash(fileName) + ") could not be received.");
			File delFile = new File(fileName);
			delFile.delete();
		}
	}
}
