package view;

import javax.swing.*;

import dht.DHTNetwork;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.UnknownHostException;

import network.*;
import model.*;

public class ClientLogin {
	
	private JFrame frame;
	private Container panel;
	private JTextField username;
	private JTextField servAddress;
	private JTextField servPort;
	private JTextField presAddress;
	private JTextField presPort;
	private JLabel usernameLabel;
	private JLabel presAddressLabel;
	private JLabel presPortLabel;
	private JLabel servAddressLabel;
	private JLabel servPortLabel;
	private JButton loginButton;
	private JButton cancelButton;
	private JPanel buttonPanel;
	private JPanel infoPanel;
	private JPanel usernamePanel;
	private JPanel servAddressPanel;
	private JPanel servPortPanel;
	private JPanel presAddressPanel;
	private JPanel presPortPanel;
	private ClientView view;
	private ClientNetwork network;
	private ClientModel model;
	private DHTNetwork dht;
	private JLabel dhtInitLabel;
	private JCheckBox dhtCheckBox;
	private JLabel dhtAddressLabel;
	private JLabel dhtPortLabel;
	private JTextField dhtAddress;
	private JTextField dhtPort;
	private JPanel dhtCheckPanel;
	private JPanel dhtAddressPanel;
	private JPanel dhtPortPanel;
	private JPanel switchPanel;
	private JCheckBox switchCheckBox;
	private JLabel switchLabel;
	
	public ClientLogin()
	{
		frame = new JFrame("P2P Client");
		panel = frame.getContentPane();
		username = new JTextField(12);
		servAddress = new JTextField("127.0.0.1", 12);
		servPort = new JTextField("4506", 6);
		presAddress = new JTextField("127.0.0.1", 12);
		presPort = new JTextField("4505", 6);
		loginButton = new JButton("Login");
		cancelButton = new JButton("Quit");
		buttonPanel = new JPanel();
		infoPanel = new JPanel();
		usernameLabel = new JLabel("Username:");
		servAddressLabel = new JLabel("Server Address:");
		servPortLabel = new JLabel("Server Port");
		presAddressLabel = new JLabel("Presentation Address:");
		presPortLabel = new JLabel("Presentation Port:");
		usernamePanel = new JPanel();
		servAddressPanel = new JPanel();
		servPortPanel = new JPanel();
		presAddressPanel = new JPanel();
		presPortPanel = new JPanel();
		dhtInitLabel = new JLabel("Join an existing DHT setup?:");
		dhtCheckBox = new JCheckBox();
		dhtAddressLabel = new JLabel("DHT Node Address:");
		dhtPortLabel = new JLabel("DHT Node Port:");
		dhtAddress = new JTextField(12);
		dhtPort = new JTextField(6);
		dhtCheckPanel = new JPanel();
		dhtAddressPanel = new JPanel();
		dhtPortPanel = new JPanel();
		switchPanel = new JPanel();
		switchLabel = new JLabel("Run in DHT mode? (instead of Centralised P2P)");
		switchCheckBox = new JCheckBox();
		
		panel.setLayout(new BorderLayout());
		infoPanel.setLayout(new GridLayout(9,1));
		usernamePanel.setLayout(new FlowLayout());
		servAddressPanel.setLayout(new FlowLayout());
		servPortPanel.setLayout(new FlowLayout());
		presAddressPanel.setLayout(new FlowLayout());
		presPortPanel.setLayout(new FlowLayout());
		dhtCheckPanel.setLayout(new FlowLayout());
		dhtAddressPanel.setLayout(new FlowLayout());
		dhtPortPanel.setLayout(new FlowLayout());
		switchPanel.setLayout(new FlowLayout());
		usernamePanel.add(usernameLabel);
		usernamePanel.add(username);
		infoPanel.add(switchPanel);
		servAddressPanel.add(servAddressLabel);
		servAddressPanel.add(servAddress);
		servPortPanel.add(servPortLabel);
		servPortPanel.add(servPort);
		presAddressPanel.add(presAddressLabel);
		presAddressPanel.add(presAddress);
		presPortPanel.add(presPortLabel);
		presPortPanel.add(presPort);
		dhtCheckPanel.add(dhtInitLabel);
		dhtCheckPanel.add(dhtCheckBox);
		dhtAddressPanel.add(dhtAddressLabel);
		dhtAddressPanel.add(dhtAddress);
		dhtPortPanel.add(dhtPortLabel);
		dhtPortPanel.add(dhtPort);
		infoPanel.add(usernamePanel);
		infoPanel.add(servAddressPanel);
		infoPanel.add(servPortPanel);
		infoPanel.add(presAddressPanel);
		infoPanel.add(presPortPanel);
		infoPanel.add(dhtCheckPanel);
		infoPanel.add(dhtAddressPanel);
		infoPanel.add(dhtPortPanel);
		switchPanel.add(switchLabel);
		switchPanel.add(switchCheckBox);
		dhtCheckBox.setEnabled(false);
		dhtAddress.setEnabled(false);
		dhtPort.setEnabled(false);
		presAddress.setSize(12, 1);
		buttonPanel.setLayout(new FlowLayout());
		buttonPanel.add(loginButton);
		buttonPanel.add(cancelButton);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		panel.add(infoPanel, BorderLayout.CENTER);
		frame.setSize(400,400);
		frame.setVisible(true);
		
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt)
			{
				System.exit(0);
			}
		});
		
		loginButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent evt)
			{
				network.initSocket();
				network.setPresDetails(presAddress.getText(), presPort.getText());
				if(switchCheckBox.isSelected())
				{
					if(dhtCheckBox.isSelected())
					{
						try {
							dht.initDHT(network.getClientAddress(), network.getClientPort(), false);
							dht.printDetails();
						} catch (UnknownHostException e) {
							System.exit(0);
						}
						dht.joinNetwork(dhtAddress.getText(), 
								Integer.parseInt(dhtPort.getText()));
					}
					else
					{
						try {
							dht.initDHT(network.getClientAddress(), network.getClientPort(), true);
							dht.printDetails();
						} catch (UnknownHostException e) {
							System.exit(0);
						}
					}
					view.initialise(false);
				}
				else
				{
					try {
						dht.initDHT(network.getClientAddress(), network.getClientPort(), false);
						dht.printDetails();
					} catch (UnknownHostException e) {
						System.exit(0);
					}
					model.initSession(username.getText(), servAddress.getText(), 
							servPort.getText());
					view.initialise(true);
				}
				new Thread(network).start();
				frame.dispose();
			}
		});
		
		dhtCheckBox.addActionListener(new ActionListener() {
		      public void actionPerformed(ActionEvent evt) {
		        if(dhtCheckBox.isSelected())
		        {
		        	dhtAddress.setEnabled(true);
		        	dhtPort.setEnabled(true);
		        }
		        else
		        {
		        	dhtAddress.setEnabled(false);
		        	dhtPort.setEnabled(false);
		        }
		      }
		    });
		
		switchCheckBox.addActionListener(new ActionListener() {
		      public void actionPerformed(ActionEvent evt) {
		        if(switchCheckBox.isSelected())
		        {
		        	dhtCheckBox.setEnabled(true);
		        	if(dhtCheckBox.isSelected())
			        {
			        	dhtAddress.setEnabled(true);
			        	dhtPort.setEnabled(true);
			        }
			        else
			        {
			        	dhtAddress.setEnabled(false);
			        	dhtPort.setEnabled(false);
			        }
		        	servAddress.setEnabled(false);
		        	servPort.setEnabled(false);
		        	username.setEnabled(false);
		        }
		        else
		        {
		        	dhtCheckBox.setEnabled(false);
		        	dhtAddress.setEnabled(false);
		        	dhtPort.setEnabled(false);
		        	servAddress.setEnabled(true);
		        	servPort.setEnabled(true);
		        	username.setEnabled(true);
		        }
		      }
		    });
		
		frame.addWindowListener(new java.awt.event.WindowAdapter() {
		    public void windowClosing(java.awt.event.WindowEvent windowEvent) {
		        if (JOptionPane.showConfirmDialog(frame, 
		            "Are you sure you want to exit?", "EXIT SESSION", 
		            JOptionPane.YES_NO_OPTION,
		            JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION){
		            System.exit(0);
		        }
		    }
		});
	}
	
	public void initView(ClientView view)
	{
		this.view = view;
	}
	
	
	public void initNetwork(ClientNetwork network)
	{
		this.network = network;
	}
	
	public void initModel(ClientModel model)
	{
		this.model = model;
	}
	
	public void initDHT(DHTNetwork dht)
	{
		this.dht = dht;
	}
}

